// Extract of pfmCore Library
// Paul F. McGinley | Kerobers
// 14th May 2023 16:34
// https://pfm.rip

using HarmonyLib;
using HMLLibrary;
using System;
using System.Reflection;
using UnityEngine;

public class Harvest_Bees : Mod
{
    private Harmony harmony;
    private string myModName = "Harvest_Bees";
    private string harmonyID
    {
        get { return $"rip.pfm.{myModName}"; }
    }

    public void Start()
    {
        harmony = new Harmony(harmonyID);
        harmony.PatchAll(Assembly.GetExecutingAssembly());

        Debug.Log("Mod Harvest_Bees has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmonyID);
        Debug.Log("Mod Harvest_Bees has been unloaded!");
    }

    /// <summary>
    /// Because we are using the AddItem method within this patch, it creates a loop.
    /// The loop is caused by looking at the StackTrace to see the AddItem was called from,
    /// As we are calling this function from within itself, it will still contain the StackTrace element triggering the AddItem function yet again.
    /// We need to track when we are overriding so we don't jump into the loop.
    /// </summary>
    static bool _Overriding_AddItem = false;

    [HarmonyPatch(typeof(Inventory), "AddItem", new Type[] { typeof(string), typeof(int) })]
    public class Patch_AddItemToPlayer
    {
        // Prefix as we wish to do our modifications before the main event.
        static void Prefix(ref string uniqueItemName, ref int amount)
        {
            Slot selectedHotbarSlot = RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarSlot();
            Item_Base hotbarItem = selectedHotbarSlot.GetItemBase();

            if (hotbarItem == ItemManager.GetItemByName("Jar_Honey") && Environment.StackTrace.Contains("BeeHive.HarvestYield") && !_Overriding_AddItem)
            {
                // Set to true as to not enter this if statement during our AddItem calls below.
                _Overriding_AddItem = true;

                int HoneyJar_Count = selectedHotbarSlot.itemInstance.Amount;
                UnityEngine.Debug.Log("HoneyJar_Count: " + HoneyJar_Count);

                int BeeJarToGive = HoneyJar_Count >= amount ? amount : HoneyJar_Count;
                UnityEngine.Debug.Log("BeeJarToGive: " + BeeJarToGive);

                amount -= BeeJarToGive;
                UnityEngine.Debug.Log("HoneyCombToGive: " + amount);

                RAPI.GetLocalPlayer().Inventory.RemoveItem("Jar_Honey", BeeJarToGive);
                RAPI.GetLocalPlayer().Inventory.AddItem(ItemManager.GetItemByName("Jar_Bee").UniqueName, BeeJarToGive);

                // Note, don't need to AddItem for honey ans changing the amount will allow the function to complete normally with a modified amount to return

                // Set this to false to allow regular calling of this class
                _Overriding_AddItem = false;
            }
        }
    }
}